#AR 1 covariance matrix
AR1.M <- function(series.len, rho, sigma2){
	times <- 1:series.len
	H <- abs(outer(times, times, "-"))
	Corr.M <- rho^H
	Cov.M <- Corr.M*sigma2/(1-rho^2)
	Cov.M 
}

###################ESTIMATING FUNCTION, DERIVATIVES AND OLS ESTIMATE - AR1
#obj.fun <- function(data, theta)
#	sum((data[-1]-theta*data[-q])^2)

est.func <- function(data, rho, q)
	2*sum(data[-1]*data[-q]) - 2*rho*sum(data[-q]^2)

#frst.der.est.func <- function(data, rho, q)
#	2*sum(data[-q]^2)

rho.hat <- function(data, q)
	sum(data[-1]*data[-q])/sum(data[-q]^2)



###################STANDARD BIAS REDUCTION
################### correct WHEN 
################### we use estimating function derived under AR1 autocovariance assumption and
################### simulation is under NORMALITY with AR1 autocovariance function
#bias function - computed under normality
bias.fun.ar.ols <- function(rho, q)
	-2*rho/q# (2/((q-1)^2*(rho^2-1)))*((q-2)*rho-(q-1)*rho^3+rho^(2*q-1))

#adjustment to the estimating function - computed under normality
adj.bias.function.ar.ols <- function(rho, q)
	-(2*(q-1)/(1-rho^2))*bias.fun.ar.ols(rho, q)

#adjusted estimating function
adj.est.fun.ar.ols <- function(data, rho, q)
	est.func(data, rho, q) + adj.bias.function.ar.ols(rho, q)


##WINDOWS SUBSAMPLING 
windows.sub <- function(q, Win.width=2, step=1){

	if(Win.width<=1)
		stop("windows width must be greater than 1")

	#defines how many points are going to be included in each sub window
		#in both coordinates
	win.width <- Win.width-1

	#step represents how many observations to skip in the definition
	#of successive sub-windows, i.e., the starting point of subregion S_k
	#in x-coordinate is min(S_{k-1}-x)+step 
	#in y-coordinate is  min(S_{k-1}-y)+step
	#if step==Win.width then the subregions are not overlapping

	#create the whole grid (see the setting in Bevilacqua 2012)
	Min <- 1
	Max <- q
	lst.x <- list()	

	if(Min+win.width<=Max){
		lst.x[[1]] <- Min:(Min+win.width)
	}else{
	stop("maximum exeeded")
	}


	cond <- 1
	ite <- 2
	while(cond){

		if(Min+win.width+step <= Max){
			lst.x[[ite]] <- (Min+step):(Min+win.width+step)
		}else{
			cond <- 0
		}

		if(cond){
			Min <- min(lst.x[[ite]])
			ite <- ite + 1
		}
	}

	lst.x
}


K.fun <- function(win.lst, q)
	q/length(unlist(win.lst))

	
#computes the estimates obtained by using the trace adjustment (see handwritten calculations)
rho.hat.RBM <- function(data, q, subs.id, S1, S2, alpha){

	K <- K.fun(subs.id, q)	
	S1w <- unlist(lapply(subs.id, function(u) sum(data[u][-length(data[u])]*data[u][-1]) ))
	S2w <- unlist(lapply(subs.id, function(u) sum((data[u][-length(data[u])])^2) ))
	i.rbm <- (S1*S2 + K*sum(S1w*S2w) )/(S2^2 + K*sum(S2w^2))
	ols.est <- S1/S2
	e.rbm <- ols.est*(1 - K*sum(S2w^2)/S2^2) + K*sum(S1w*S2w)/S2^2
	
	list(implicit.rbm=i.rbm, explicit.rbm=e.rbm)

}



#for the implementation of the following function, see the help page 
#of function tsboot (in library(boot))
model.based.bootstrap <- function(data, B=500){

	rho.hat <- function(data){
		q <- length(data)
		sum(data[-1]*data[-q])/sum(data[-q]^2)
	}

	q <- length(data)
	ols.ar <- rho.hat(data)
	#see help page tsboot
	##begin
  ols.model <- list(order = c(1, 0, 0), ar = ols.ar)
  ols.res <- data[-1]-ols.ar*data[-q]
  ols.res <- ols.res[-1]
  ols.res <- ols.res - mean(ols.res)
     
  model.based.sim <- function(res,n.sim, ran.args) {
        # random generation of replicate series using arima.sim 
        rg1 <- function(n, res) sample(res, n, replace = TRUE)
        ts.orig <- ran.args$ts
        ts.mod <- ran.args$model
        mean(ts.orig)+ts(arima.sim(model = ts.mod, n = n.sim,
                         rand.gen = rg1, res = as.vector(res)))
  }
     
  tmp <- tsboot(ols.res, rho.hat, R = B, sim = "model", n.sim = q,
                   orig.t = FALSE, ran.gen = model.based.sim, 
                   ran.args = list(ts = data, model = ols.model))
  ##end
     
	#bias corrected estimate
	2*rho.hat(data) - mean(tmp$t)			
}


stationary.bootstrap <- function(data, B=500, alpha){
	win.len <- round(length(data)^alpha)
	tmp <- sapply(win.len, function(u) tsboot(data, rho.hat, q=length(data), R = B, l = u, sim = "geom")$t)

	#bias corrected estimate
	if(length(win.len)>1){
		2*rho.hat(data, length(data)) - colMeans(tmp)
	}else{
		2*rho.hat(data, length(data)) - mean(tmp)
	}
}

##to be deleted on final version of the code. use the one above
# stationary.bootstrap <- function(data, B=500, alpha){
# 	win.len <- round(length(data)^alpha)
# 	tmp <- sapply(win.len, function(u) tsbootstrap(data, statistic=rho.hat, q=length(data), 
# 		nb = B, type="stationary", b=u))

# 	#bias corrected estimate
# 	if(length(win.len)>1){
# 		rho.hat(data, length(data)) - apply(tmp, 2, function(x) x$bias)
# 	}else{
# 		rho.hat(data, length(data)) - tmp$bias
# 	}
# }


jackknife <- function(data){

	#split the sample in two
	q <- length(data)
	OLS <- rho.hat(data, q)
	OLS1 <- rho.hat(data[1:(q/2)], q/2)
	OLS2 <- rho.hat(data[(q/2+1):q], q/2)

	2*OLS - (OLS1+OLS2)/2
}




estimates.ols.miss.distr <- function(data, q, lst.subs.id, methods, alpha, ud, df, mu, sigma2, 
																		 B=500, all.methods=TRUE){

	S1 <- sum(data[-1]*data[-q])
	S2 <- sum(data[-q]^2)

	#OLS
	ols.e <- S1/S2
	#STANDARD BR (Firth's)
	br <- nleqslv(x=ols.e, function(o) adj.est.fun.ar.ols(data, o, q))$x
	if(all.methods){
		#jackknife
		jackk <- jackknife(data)
		#RBM (implict and explicit)
		rbm <- t(sapply(lst.subs.id, function(u) rho.hat.RBM(data, q, u, S1, S2)))
		#model based bootstrap
		if(abs(ols.e)<1){
			ols.boot.bc.mb <- model.based.bootstrap(data, B)
		}else{
			ols.boot.bc.mb <- 0
		}
		#stationary bootstrap
		ols.boot.bc.s <- stationary.bootstrap(data, B, alpha)
	}else{
		ols.boot.bc.mb <- jackk <- 0
		ols.boot.bc.s <- rep(0, length(alpha))
		rbm <- rep(0, 2*length(lst.subs.id))
	}
	length.ols.br.jackk.mboot <- 4 
	len.robust.br <- length(rbm)
	len.boot.sb <- length(ols.boot.bc.s)

	pars <- rep("rho", length.ols.br.jackk.mboot + len.robust.br + len.boot.sb)

	data.frame(parameter = pars,
						 method = methods,
						 estimate = c(ols.e, br, as.numeric(rbm), ols.boot.bc.mb, ols.boot.bc.s, jackk),
						 stationary=ifelse(abs(ols.e)>=1, 0, 1))
}
