#To be used when teh error distribution is misspecified 
###Under.Distr: error distribution. Possible values are "normal", "t", "laplace", as describe in the paper
###n_cores: number of cores to run simulations in parallel
###base_n: smallest samples size to be considered in simulations
###path: where the resuklts are going to be saved
###theta: autocorrelation parameter
###alpha.rbm: used to compute the windows/blocks length for RBM estimators
###alpha.boot: used to compute the windows/blocks average length for stationary bootstrap
####note that to this vector is going to be added the entry log(T/2)/log(n) in order to have 
####blocks of average length equal to half the sample size 
###verbose: print some output in order to see how simulations are progressing
###print.every: if verbose=TRUE, then the output is printed on screen after print.every iterations are done
###all.methods: if you want to replicate the simulation study in the paper
####it is sufficient to compute all estimators just for one underlying distribution (see comment in the paper)
####and then set all.methods=FALSE for the remaining distributions in order to just compute Firth's 
####bias reduced estimator  
simu.func.ols.ar.miss.distr <- function(Under.Distr, n_cores=5, base_n=5e1, 
										path=".", theta=c(0.2, 0.5, 0.9), 
										alpha.rbm=c(1/3, 1/2), 
										alpha.boot=c(1/3), 
										verbose=TRUE, print.every=1e3, all.methods=TRUE){

	if(!all(Under.Distr%in%c("normal", "t", "laplace")))
		stop("Under.Distr must be ``normal'', ``t'', ``laplace''")

	mu <- 0
	sigma2 <- 1
	df <- 5

	replications <- 1
	RHO <- theta

	for(ud in Under.Distr){

	for(rho in RHO){

	multiplier <- 2^(0:4)
	#sample sizes
	n_obs <- base_n * multiplier
	#number of Monte Carlo trials for each sample size
	n_simu <- round((250*(2^16)/n_obs))
	if(verbose){
		print("Sample sizes in the simulation study")
		print(n_obs)
		print("Number of Monte Carlo trials for each sample size in the simulation study")
		print(n_simu)
	}	

	N.split <- rep(1, length(n_simu))
	set.seed(1)
	seeds.urn <- round(runif(sum(N.split), 0, 10^7))

	seeds <- vector(mode="list", len=length(n_simu))
	for(i in 1:length(n_simu)){
		sampled <- sample(seeds.urn, N.split[[i]], replace=FALSE) 
		seeds[[i]] <- matrix(sampled, nr=N.split[[i]], nc=1)
		seeds.urn <- setdiff(seeds.urn, sampled)
	}

	repl <- 1
	for (j in seq_along(n_simu)) {

			Cov.M <- AR1.M(n_obs[j], rho, sigma2)

			if(ud=="t"){
				CholCov.M <- chol(Cov.M*(df-2)/df)
			}else{
				CholCov.M <- chol(Cov.M)
			}

			MEAN <- rep(mu, n_obs[j])
			
			#windows subsampling
			alpha.boot.in <- c(alpha.boot, log(n_obs[j]/2)/log(n_obs[j]))
			WIN1 <- windows.sub(n_obs[j], Win.width=round(n_obs[j]^alpha.rbm[1]), 
				step=round(n_obs[j]^alpha.rbm[1]))
			WIN2 <- windows.sub(n_obs[j], Win.width=round(n_obs[j]^alpha.rbm[2]), 
				step=round(n_obs[j]^alpha.rbm[2]))
			attr(WIN1,"alpha") <- alpha.rbm[1]
			attr(WIN2,"alpha") <- alpha.rbm[2]
			##to be removed in the final versions
			
			LST.WIN <- list(WIN1, WIN2)

			if(ud!="normal"){
				methods <- c("ols", "br.misspecification",
										paste0("I.rbm.alpha", alpha.rbm), 
										paste0("E.rbm.alpha", alpha.rbm), 
										"ols.boot.mb", 
										paste0("ols.boot.s.alpha", c(alpha.boot.in[1],".n/2")),
										"jackk")
			}else{
				methods <- c("ols", "br",
										paste0("I.rbm.alpha", alpha.rbm), 
										paste0("E.rbm.alpha", alpha.rbm), 
										"ols.boot.mb", 
										paste0("ols.boot.s.alpha", alpha.boot.in),
										"jackk")
			}

			for(r in repl){
						results <- c()
						n.simu.split <- n_simu[j]/N.split[j]
						for(u in 1:N.split[j]){
						set.seed(seeds[[j]][u, r])

#						cat("start data simulation-")		
							if(ud=="t"){
								#multivariate t
								simu.data <- rmvt(n.simu.split, MEAN, CholCov.M, df=df, ncores = n_cores, isChol = TRUE)
							}else{
								if(ud=="laplace"){
									#symmetric multivariate laplace				
									simu.data <- rmvn(n.simu.split, MEAN, CholCov.M, ncores = n_cores, isChol = TRUE)*rexp(n.simu.split)
								}else{
									#multivariate normal
									simu.data <- rmvn(n.simu.split, MEAN, CholCov.M, ncores = n_cores, isChol = TRUE)
								}
							}
#						cat("done\n")
						res <- mclapply(seq.int(n.simu.split), function(i){
							if(verbose){
								if (i %% print.every == 0){
						    		cat("DISTR =",ud, "rho =",rho,"n =", n_obs[j], "| replication", r, "| sample", i+(u-1)*n.simu.split, "/", n_simu[j], "\n")
						    	}
							}		
								out <- estimates.ols.miss.distr(simu.data[i,], n_obs[j], 
									LST.WIN, methods, alpha=alpha.boot.in, 
									ud=ud, df=df, mu=mu, sigma2=sigma2,
									all.methods=all.methods)
								out$sample <- i+(u-1)*n.simu.split
								out
						}, mc.cores = n_cores)
							rm(simu.data)
						results.tmp1 <- rbindlist(res)
							rm(res)
#						cat("start rbind whole-")
							results <- rbind(results, results.tmp1)
#						cat("done\n")
							rm(results.tmp1)
						}
					
					results$true_value <- rho#rep(rho, 2 + length(LST.WIN))
					attr(results, "replication") <- r
					attr(results, "N") <- n_simu[j]
					attr(results, "n") <- n_obs[j]

					if(ud=="t")
						attr(results, "df") <- df
					
					if(ud=="t"){
						save(list = "results",
								 file = paste0(path, "/_",rho,"_OLS-AR1-T-", j, "-repl-", r, ".rda"))
					}else{
						if(ud=="laplace"){
							save(list = "results",
									 file = paste0(path, "/_",rho,"_OLS-AR1-LAPLACE-", j, "-repl-", r, ".rda"))
						}else{
							save(list = "results",
									 file = paste0(path, "/_",rho,"_OLS-AR1-NORMAL-", j, "-repl-", r, ".rda"))	    
						}
					}
					rm(results)
				}
			}
		}
	}
}



outputs.ar1 <- function(Under.Distr=c("normal", "t", "laplace"), theta){ 

	if(!all(Under.Distr%in%c("normal", "t", "laplace")))
		stop("Under.Distr must be ``normal'', ``t'', ``laplace''")

suppressMessages({

	require("dplyr")
	ud <- Under.Distr

	path="."
	files_whole <- dir(path, full.names = TRUE)


	if(ud=="t"){
			files <- files_whole[grep(paste0("_",theta,"_OLS-AR1-T-"), files_whole)]
	}else{
		if(ud=="laplace"){
			files <- files_whole[grep(paste0("_",theta,"_OLS-AR1-LAPLACE-"), files_whole)]
		}else{
			files <- files_whole[grep(paste0("_",theta,"_OLS-AR1-NORMAL-"), files_whole)]
		}
	}
	
	Nsam <- length(files)

	files_whole <- files

	for(i in 1:Nsam){
#		cat(i,"\n")
		
		if(ud=="t"){
			files <- files_whole[grep(paste0("_",theta,"_OLS-AR1-T-",i), files_whole)]
		}else{
			if(ud=="laplace"){
				files <- files_whole[grep(paste0("_",theta,"_OLS-AR1-LAPLACE-",i), files_whole)]
			}else{
				files <- files_whole[grep(paste0("_",theta,"_OLS-AR1-NORMAL-",i), files_whole)]
			}
		}

		all_results <- NULL
		for (f in 1:length(files)) {
#				cat(f,"")
				load(files[f])
				cres <- results

				cres$n <- attr(cres, "n")
				cres$N <- attr(cres, "N")
				cres$replication <- attr(cres, "replication")	
				all_results <- cres

		summaries.full.tmp <- all_results %>% group_by(n, parameter, method, stationary) %>%
				summarize(samples_perc = 100 * n() / (max(replication) * unique(N)),
							    bias = mean(estimate - true_value),
							    variance = var(estimate),
							    mc_error = sd(estimate)/sqrt(unique(N)),
							    scaled_bias = bias * unique(n),
							    scaled_bias_mc_error = mc_error * unique(n),
		 						mse = var(estimate) + mean(estimate - true_value)^2,
		 						prop.non.stat = 1-length(estimate)/unique(N))

			rm(all_results)

			if(f>1){
				summaries.full.tmp[,-c(1:4)] <- summaries.full.tmp[,-c(1:4)]*(1/f) + summaries.full[,-c(1:4)]*(f-1)/f 
				summaries.full <- summaries.full.tmp
			}else{
				summaries.full <- summaries.full.tmp
			}
			rm(summaries.full.tmp)
		}

		summaries.full$samples_perc <- 100

		if(i>1){
			summaries.whole <- rbind(summaries.whole, summaries.full)		
		}else{
			summaries.whole <- summaries.full
		}
		rm(summaries.full)
#		cat("\n\n")
	}

	res <- summaries.whole
	rm(summaries.whole)

	ids <- grep("ols.boot.s.alpha0.8", res$method)
	res$method[ids] <- "ols.boot.s.n/2"

	RES <- res

	stationarity <- unique(RES$stationary)


	method.names.paper <-  c(
		"$\\theta^{*}$", #1
		"$\\theta^{\\dagger}_{1/3}$", #2
		"$\\theta^{\\dagger}_{1/2}$", #3
		"$\\tilde{\\theta}_{1/3}$", #4
		"$\\tilde{\\theta}_{1/2}$", #5
		"$\\hat{\\theta}^J$", #6
		"$\\hat{\\theta}$", #7
		"$\\hat{\\theta}^M$",#8
		"$\\hat{\\theta}^S_{1/3}$", #9
		"$\\hat{\\theta}^S_{T/2}$" #10
		)


	for(i in 1:length(stationarity)){
		res <- RES %>% filter(stationary==unique(RES$stationary)[i])

		n_obs <- sort(unique(res$n))
		n.method <- length(levels(factor(res$method)))
		res.out.b <- res%>% select(method, n, bias) %>% select(bias)
		res.out.b <- matrix(res.out.b$bias*100,nr=n.method, byr=FALSE)
		rownames(res.out.b) <- method.names.paper#res$method[1:n.method]
		colnames(res.out.b) <- sort(n_obs)

		res.out.v <- res%>% select(method, n, variance) %>% select(variance)
		res.out.v <- matrix(res.out.v$variance,nr=n.method, byr=FALSE)
		rownames(res.out.v) <- method.names.paper#res$method[1:n.method]
		colnames(res.out.v) <- sort(n_obs)

		df.slopes <- log(abs(res.out.b ))
		slopes <- sapply(1:n.method, function(u) lm(df.slopes[u,]~log(n_obs))$coefficients[2])
		names(slopes) <- rownames(res.out.b)

		ids.na <- which(apply(res.out.v, 1, sum)==0)
		res.out.v[ids.na,] <- res.out.b[ids.na,] <- slopes[ids.na] <- NA

		prop.non.stat <- matrix(res$prop.non.stat,nr=n.method, byr=FALSE)

		

		if(stationarity[i]==1){

			res_stationary <- list(bias=res.out.b, variance=res.out.v, slopes=slopes,
				mc_error_range=format(range(res$mc_error[res$mc_error>0]),scientific=TRUE))
		}else{
			res_nonstationary <- list(bias=res.out.b, variance=res.out.v, slopes=slopes, 
				mc_error_range=format(range(res$mc_error[res$mc_error>0]),scientific=TRUE))
		}
	}
})
	if(length(stationarity)>1){
		prop.non.stat.nms <- prop.non.stat[1,]
		names(prop.non.stat.nms) <- n_obs
		cat("Summaries computed conditional on stationary estimated models.\n")
		cat("The proportion of non stationary estimated models for each sample size is \n\n")
		print(prop.non.stat.nms)
		cat("\n")
		list(res_stationary=res_stationary, res_nonstationary=res_nonstationary, prop.non.stat=prop.non.stat[1,])
	}else{
		res_stationary
	}
}


tables.ar1 <- function(theta){

	out <- list()

	out[[1]] <- suppressMessages(try(outputs.ar1(Under.Distr="normal", theta=theta), silent=TRUE))
	out[[2]] <- suppressMessages(try(outputs.ar1(Under.Distr="t", theta=theta), silent=TRUE))
	out[[3]] <- suppressMessages(try(outputs.ar1(Under.Distr="laplace", theta=theta), silent=TRUE))

	outs <- rep(0,3)
	distrib <- c("normal","t","laplace")
	df.out <- c()
	for(i in 1:3){	
		if(class(out[[i]])!="try-error"){
			outs[i] <- 1
		}else{
			warning(paste0("Results for theta=", theta," and for the ", distrib[i], " not found in folder\n"))
		}
		
		if(exists("res_stationary", out[[i]])){
			df.out <- rbind(df.out, out[[i]]$res_stationary$bias)
		}else{
			df.out <- rbind(df.out, out[[i]]$bias)	
		}
	}	

	#create a variable containing the rownames
	rownms <-  rep(c(
		"$\\theta^{*}$", #1
		"$\\theta^{\\dagger}_{1/3}$", #2
		"$\\theta^{\\dagger}_{1/2}$", #3
		"$\\tilde{\\theta}_{1/3}$", #4
		"$\\tilde{\\theta}_{1/2}$", #5
		"$\\hat{\\theta}^J$", #6
		"$\\hat{\\theta}$", #7
		"$\\hat{\\theta}^M$",#8
		"$\\hat{\\theta}^S_{1/3}$", #9
		"$\\hat{\\theta}^S_{T/2}$" #10
		), sum(outs))


	df.out <- data.frame(row=rownms, data.frame(df.out))

	require(xtable)
	print(xtable(
		#swap rows to match the displyed order in the table in the main text
		df.out[c(7, 1, 4:5, 2:3, 8, 6, 9:10, 11, 21),]),
	include.rownames=FALSE, sanitize.text.function = identity)
}