## Copyright (C) 2022 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!
##
## It is assumed that the user has a Julia and R installation with all
## packages required by the current script and any other script
## included.
##
##
## Code to reproduce the results in Figure 3


library("memisc")
library("dplyr")
library("ggplot2")

image_path <- "images/"

n_obs_vec <- c(75, 150, 300, 600)

load(file.path(image_path, paste0("model_selection_probit_", n_obs_vec[1], ".rda")))

ics <- NULL
biases_ml <- biases_br <- biases_tr <- biases_tr_extra <-
    mses_ml <- mses_br <- mses_tr <- mses_tr_extra <- matrix(NA, length(n_obs_vec), 5)
converged_ml <- converged_tr <- converged_tr_extra <- converged_br <- as.list(numeric(length(n_obs_vec)))
separated <- matrix(NA, length(n_obs_vec), 16)
n_separated_samples <- numeric(length(n_obs_vec))
names(converged_ml) <- names(converged_tr) <- names(converged_tr_extra) <- names(converged_br) <-
    rownames(separated) <- names(n_separated_samples) <-
    rownames(mses_ml) <- rownames(mses_br) <- rownames(mses_tr) <- rownames(mses_tr_extra) <-
    rownames(biases_ml) <- rownames(biases_br) <- rownames(biases_tr) <- rownames(biases_tr_extra) <- n_obs_vec
colnames(separated) <- rownames(res[[1]]$coef_ml)
colnames(biases_br) <- colnames(biases_ml) <- colnames(biases_tr) <- colnames(biases_tr_extra) <- paste0("parameter.beta", 0:4)

for (n_obs in n_obs_vec) {
    load(file.path(image_path, paste0("model_selection_probit_", n_obs, ".rda")))
    true_model <- paste(which(betas != 0), collapse = "-")
    nam <- as.character(n_obs)
    sep_mat <- do.call("rbind", lapply(res, "[[", "separated"))
    separated[nam, ] <- colSums(sep_mat, na.rm = TRUE)
    n_separated_samples[nam] <- sum(apply(sep_mat, 1, any, na.rm = TRUE))
    converged_ml[[nam]] <- sapply(res, "[[", "converged_ml")
    converged_br[[nam]] <- sapply(res, "[[", "converged_br")
    converged_tr[[nam]] <- sapply(res, "[[", "converged_tr")
    converged_tr_extra[[nam]] <- sapply(res, "[[", "converged_tr_extra")
    c_ml <- sapply(res, function(x) x$coef_ml[true_model, ])
    c_tr <- sapply(res, function(x) x$coef_tr[true_model, ])
    c_tr_extra <- sapply(res, function(x) x$coef_tr_extra[true_model, ])
    c_br <- sapply(res, function(x) x$coef_br[true_model, ])
    biases_ml[nam, ] <- rowMeans(c_ml[, ]) - betas
    biases_tr[nam, ] <- rowMeans(c_tr[, !sep_mat[, true_model]]) - betas
    biases_tr_extra[nam, ] <- rowMeans(c_tr_extra) - betas # rowMeans(c_tr_extra[, !sep_mat[, true_model]]) - betas
    biases_br[nam, ] <- rowMeans(c_br) - betas # rowMeans(c_br[, !sep_mat[, true_model]]) - betas
    mses_ml[nam, ] <- rowMeans((c_ml[, !sep_mat[, true_model]] - betas)^2)
    mses_tr[nam, ] <- rowMeans((c_tr[, !sep_mat[, true_model]] - betas)^2)
    mses_tr_extra[nam, ] <- rowMeans((c_tr_extra[, !sep_mat[, true_model]] - betas)^2)
    mses_br[nam, ] <- rowMeans((c_br[, !sep_mat[, true_model]] - betas)^2)
    ml_selections <- sapply(res, "[[", "model_ml")
    br_selections <- sapply(res, "[[", "model_br")
    pml_selections <- sapply(res, "[[", "model_tr")
    pml_extra_selections <- sapply(res, "[[", "model_tr_extra")
    all_models <- rownames(res[[1]]$coef_ml)
    tic_ml <- tic_br <- tic_tr <- tic_tr_extra <- numeric(length(all_models))
    names(tic_ml) <- names(tic_br) <- names(tic_tr) <- names(tic_tr_extra) <- all_models
    out_ml <- table(ml_selections["tic", ])
    out_br <- table(br_selections["tic", ])
    out_tr <- table(pml_selections["tic", ])
    out_tr_extra <- table(pml_extra_selections["tic", ])
    tic_ml[names(out_ml)] <- out_ml
    tic_br[names(out_br)] <- out_br
    tic_tr[names(out_tr)] <- out_tr
    tic_tr_extra[names(out_tr_extra)] <- out_tr_extra
    tics <- data.frame(selection_prob = c(tic_ml, tic_br, tic_tr, tic_tr_extra)/n_simu,
                       model = rep(all_models, 4),
                       method = rep(factor(c("ML", "AS", "RBM", "RBMp"),
                                           ordered = TRUE, levels = c("ML", "AS", "RBM", "RBMp")),
                                    each = c(length(all_models))),
                       ic = "TIC",
                       n = n_obs)
    all_models <- rownames(res[[1]]$coef_ml)
    aic_ml <- aic_br <- aic_tr <- aic_tr_extra <- numeric(length(all_models))
    names(aic_ml) <- names(aic_br) <- names(aic_tr) <- names(aic_tr_extra) <- all_models
    out_ml <- table(ml_selections["aic", ])
    out_br <- table(br_selections["aic", ])
    out_tr <- table(pml_selections["aic", ])
    out_tr_extra <- table(pml_extra_selections["aic", ])
    aic_ml[names(out_ml)] <- out_ml
    aic_br[names(out_br)] <- out_br
    aic_tr[names(out_tr)] <- out_tr
    aic_tr_extra[names(out_tr_extra)] <- out_tr_extra
    aics <- data.frame(selection_prob = c(aic_ml, aic_br, aic_tr, aic_tr_extra)/n_simu,
                       model = rep(all_models, 4),
                       method = rep(factor(c("ML", "AS", "RBM", "RBMp"),
                                           ordered = TRUE,
                                           levels = c("ML", "AS", "RBM", "RBMp")),
                                    each = c(length(all_models))),
                       ic = "AIC",
                       n = n_obs)
    ics <- rbind(ics, aics, tics)
}

tolong <- function(mat, method) {
    mat <- reshape(as.data.frame(cbind(mat, n = n_obs_vec)),
                   idvar = "n", varying = 1:5,
                   v.names = "estimate", direction = "long")
    names(mat) <- c("n", "parameter", "estimate")
    mat %>% mutate(parameter = dplyr::recode(parameter,
                                      `1` = "$\\beta_0$", `2` = "$\\beta_1$", `3` = "$\\beta_2$",
                                      `4` = "$\\beta_3$", `5` = "$\\beta_4$"),
                   method = method)
}

## reshape to long format, ready for xtabs
bias <- rbind(tolong(biases_ml, "ML"),
              tolong(biases_br, "AS"),
              tolong(biases_tr, "RBM"),
              tolong(biases_tr_extra, "RBMp"))
bias$method <- factor(bias$method, ordered = TRUE, levels = c("ML", "AS", "RBM", "RBMp"))
mses <- rbind(tolong(mses_ml, "ML"),
              tolong(mses_br, "AS"),
              tolong(mses_tr, "RBM"),
              tolong(mses_tr_extra, "RBMp"))
mses$method <- factor(mses$method, ordered = TRUE, levels = c("ML", "AS", "RBM", "RBMp"))
bias$quantity <- "Bias"
mses$quantity <- "MSE"
results <- rbind(bias, mses)
with(results,
     toLatex(ftable(xtabs(estimate ~ quantity + method + parameter + n)),
             useDcolumn = FALSE, digits = 3)
     ) %>% clipr::write_clip()

## Order models by number of covariates
model_names <- unique(ics$model)
model_names <- model_names[order(nchar(model_names))]
ics$model <- factor(ics$model, levels = model_names, labels =  gsub("-","",model_names), ordered = TRUE)

pdf(file.path(image_path, "info_crit.pdf"), width = 8, height = 7)
ggplot(ics) +
    geom_col(aes(x = model, y = selection_prob * 100, colour = NULL, fill = ic), position = "dodge") +
    theme_minimal() +
    theme(legend.position = "none") +
    labs(x = "model", y = "selection proportion", labeller = label_parsed)  +
    scale_colour_grey() +
    scale_fill_grey() +
    facet_grid(method ~ n) +
    coord_flip()
dev.off()


