## Copyright (C) 2022 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!
##
## It is assumed that the user has a Julia and R installation with all
## packages required by the current script and any other script
## included.
##
##
## Code to reproduce the simulation study in Section S8

setting_scripts <- c("arxiv-2010.13687-S6.R")
                    "arxiv-2204.07907-SJ4-I.R",
                    "arxiv-2204.07907-SJ4-II.R",
                    "arxiv-2204.07907-SJ4-III.R")
ncores <- 10

library("optimx")
library("MASS")
library("brglm2")
library("parallel")

path <- "."

for (setting_script in setting_scripts) {
    source(file.path(path, "negbin-functions.R"))
    source(file.path(path, setting_script))
    img_name <- paste0("negbin-results-", "n", n, "-p", length(betas), "-kappa", kappa, ".rda")

    omethods <- c("L-BFGS-B", "nlminb", "nlm", "CG")

    res_ml <- mclapply(seq.int(n_simu), function(j) {
        out <- fit_ml(sim_Y[, j], X, pars0, tr, kappa_bd, se_bd, omethods)
        if (has_failed(out, gr_bd = 0.1)) {
            out$estimate <- NA
        }
        if (j %% 100 == 0) cat("ML:", j, "\n")
        rownames(out) <- names(pars0)
        out$sample <- j
        out
    }, mc.cores = ncores)

    save(res_ml, omethods, img_name, file = file.path(path, img_name))

    res_erbm <- mclapply(seq.int(n_simu), function(j) {
        out <- fit_erbm(sim_Y[, j], X, res_ml[[j]]$estimate, tr, kappa_bd, se_bd)
        if (has_failed(out, gr_bd = 0.1)) {
            out$estimate <- NA
        }
        rownames(out) <- names(pars0)
        out$sample <- j
        if (j %% 100 == 0) cat("eRBM:", j, "\n")
        out
    }, mc.cores = ncores)

    save(res_ml, res_erbm, omethods, img_name, file = file.path(path, img_name))

    res_irbm <- mclapply(seq.int(n_simu), function(j) {
        out <- fit_irbm(sim_Y[, j], X, res_erbm[[j]]$estimate, tr, kappa_bd, se_bd, omethods)
        if (has_failed(out, gr_bd = 0.1)) {
            out <- fit_irbm(sim_Y[, j], X, pars0, tr, kappa_bd, se_bd, omethods)
        }
        if (has_failed(out, gr_bd = 0.1)) {
            out$estimate <- NA
        }
        rownames(out) <- names(pars0)
        out$sample <- j
        if (j %% 100 == 0) cat("iRBM:", j, "\n")
        out
    }, mc.cores = ncores)

    save(res_ml, res_erbm, res_irbm, omethods, img_name, file = file.path(path, img_name))

    res_br <- mclapply(seq.int(n_simu), function(j) {
        out <- fit_br(sim_Y[, j], X, res_irbm[[j]]$estimate, tr, kappa_bd, se_bd)
        if (has_failed(out, gr_bd = 0.1)) {
            out <- fit_br(sim_Y[, j], X, pars0, tr, kappa_bd, se_bd)
        }
        if (has_failed(out, gr_bd = 0.1)) {
            out$estimate <- NA
        }
        rownames(out) <- names(pars0)
        out$sample <- j
        if (j %% 100 == 0) cat("BR:", j, "\n")
        out
    }, mc.cores = ncores)

    save(res_ml, res_erbm, res_irbm, res_br, omethods, img_name, file = file.path(path, img_name))

}
